/*
 * Game world module
 * Copyright 2017 William Lupshenko
 * 
 * This file is a part of TUServer
 * 
 * TUServer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version
 */

var
	fs = require("fs");

module.exports = {
	//Secondary variables
		players: {
			//Secondary variables
				defaultGuiData: {},
				
				list: {},
				sounds: {},
			
			//Methods
				New: function() {
					var spawnPoint = module.exports.players.findSpawnPoint();
					
					Object.assign(this, 
						new game.realms.gameWorld.objectTypes.player.New(),
						
						{
							x: spawnPoint.x,
							y: spawnPoint.y,
							
							currentGuiType: "default",
							guiData: JSON.parse(JSON.stringify(
								module.exports.players.defaultGuiData
							))
						}
					);
				},
				
				findSpawnPoint: function() {
					for (
						var x = game.realms.gameWorld.playerOffset.x + 1;
						x <
							game.config.list.gameWorld.size -
							game.realms.gameWorld.playerOffset.x - 1;
						x++
					) {
						for (
							var y = game.realms.gameWorld.playerOffset.y + 1;
							y <
								game.config.list.gameWorld.size -
								game.realms.gameWorld.playerOffset.y - 1;
							y++
						) {
							if (!game.config.list.gameWorld.objects[x][y]) {
								return {
									x: x,
									y: y
								};
							}
						}
					}
					
					return {
						x: NaN,
						y: NaN
					};
				},
				
				setCurrent: function(player) {
					this.current = player;
					
					game.config.list.gameWorld.playerPosition.x =
						this.list[player].x;
					game.config.list.gameWorld.playerPosition.y =
						this.list[player].y;
					
					game.config.list.gameWorld.cameraPosition.x =
						this.list[player].x - game.realms.gameWorld.playerOffset.x;
					game.config.list.gameWorld.cameraPosition.y =
						this.list[player].y - game.realms.gameWorld.playerOffset.y;
					
					game.realms.gameWorld.currentGuiType.data =
						this.list[player].guiData[
							this.list[player].currentGuiType
						];
				},
				update: function(player) {
					this.list[player].x =
						game.config.list.gameWorld.cameraPosition.x +
						game.realms.gameWorld.playerOffset.x;
					this.list[player].y =
						game.config.list.gameWorld.cameraPosition.y +
						game.realms.gameWorld.playerOffset.y;
				},
				
				connect: function(player) {
					this.sounds[player] = [];
					
					Object.defineProperty(this.list, player, {
						configurable: true,
						enumerable: true,
						
						get: new Function("",
							"return game.config.list.players[" +
								"\"" + player + "\"" +
							"];"
						),
						set: new Function("value",
							"game.config.list.players[" +
								"\"" + player + "\"" +
							"] = value;"
						)
					});
					
					if (!game.config.list.players[player]) {
						this.list[player] = new this.New();
					}
					
					game.config.list.gameWorld.objects[
						this.list[player].x
					][
						this.list[player].y
					] = this.list[player];
				},
				disconnect: function(player) {
					this.list[player].currentGuiType = "default";
					
					delete this.sounds[player];
					
					game.config.list.gameWorld.objects[
						this.list[player].x
					][
						this.list[player].y
					] = new game.realms.gameWorld.objectTypes.grave.New();
					
					delete this.list[player];
				}
		},
	
	//Methods
		run: function() {
			this.init();
			
			this.update();
		},
		
		init: function() {
			//Prepare for virtual client running
				window = {
					setTimeout: function() {},
					setInterval: function() {}
				};
				game = {
					screen: {
						width: 320,
						height: 240
					},
					
					textures: {},
					sounds: {},
					
					locales: {},
					get currentLocale() {
						return this.locales[
							this.config.list.main.locale
						] || this.locales[
							this.config.defaultList.main.locale
						];
					},
					
					realms: {},
					get currentRealm() {
						return this.realms.gameWorld;
					}
				};
				
				alert = function() {
					console.log.apply(null, arguments);
				};
				
				function loadScript(path) {
					try {
						new Function(
							"fs",
							"win",
							
							fs.readFileSync(path, "utf-8")
						)(
							fs,
							{}
						);
					} catch(e) {
						console.log(new ReferenceError(
							"can't load " +
							path +
							
							" (" + e + ")"
						));
					}
				}
			
			//Load engine
				loadScript("client/engine/config.js");
				loadScript("client/engine/hotkeys.js");
				loadScript("client/engine/main_loadContent.js");
				
				game.loadTexture = function() {
					return {};
				};
				game.loadSound = function() {
					return {
						play: function() {
							module.exports.players.sounds[
								module.exports.players.current
							].push(
								this.sound
							);
						}
					};
				};
			
			//Load content
				//Scripts
					game.contentFolders =
						game.contentFolders.map(function(currentValue) {
							return "client/" + currentValue;
						});
					
					game.loadContent();
					
					game.realms.gameWorld.fail = function(object) {
						Object.assign(object,
							new game.realms.gameWorld.objectTypes.player.New(),
							
							{
								currentGuiType: "default"
							}
						);
					};
					Object.defineProperty(game.realms.gameWorld, "currentGuiType", {
						configurable: true,
						enumerable: true,
						
						get: function() {
							return this.guiTypes[
								module.exports.players.list[
									module.exports.players.current 
								].currentGuiType
							];
						},
						set: function(value) {
							module.exports.players.list[
								module.exports.players.current 
							].currentGuiType = Object.keys(
								this.guiTypes
							)[Object.values(
								this.guiTypes
							).indexOf(value)];
						}
					});
				
				//Textures, sounds, other shit
					for (var i in game.textures) {
						game.textures[i] = i;
					}
					for (var i in game.sounds) {
						game.sounds[i].sound = i;
					}
				
			//Read config
				game.config.pathPrefix = "client/" + game.config.pathPrefix;
				
				Object.assign(game.config.defaultList, {
					players: {}
				});
				Object.assign(game.config.paths, {
					players: "players.json"
				});
				
				game.config.read();
			
			//Load map
				if (game.config.list.gameWorld.objects.length == 0) {
					game.realms.gameWorld.generator.run(
						game.config.list.gameWorld
					);
					
					game.config.list.gameWorld.objects[
						game.config.list.gameWorld.playerPosition.x
					][
						game.config.list.gameWorld.playerPosition.y
					] = null;
					
					game.config.write("gameWorld");
					game.config.write("players");
				}
			
			//Generate default GUI data
				for (var i in game.realms.gameWorld.guiTypes) {
					this.players.defaultGuiData[i] =
						game.realms.gameWorld.guiTypes[i].defaultData;
				}
		},
		
		update: function() {
			for (var i in module.exports.players.list) {
				module.exports.players.setCurrent(i);
				
				game.realms.gameWorld.updateChunk();
				
				module.exports.players.update(i);
			}
			
			setTimeout(module.exports.update, 20);
		},
		
		handleHotkey: function(player, keyCode, isDown) {
			this.players.setCurrent(player);
			
			[
				game.hotkeys.released,
				game.hotkeys.pressed
			][+isDown].push(keyCode);
			game.realms.gameWorld.currentGuiType.handleHotkeys();
			
			this.players.update(player);
		},
		
		render: function(player) {
			this.players.setCurrent(player);
			
			var layers = game.realms.gameWorld.update();
			
			this.players.update(player);
			
			return layers;
		}
};
