/*
 * Socket module
 * Copyright 2017 William Lupshenko
 * 
 * This file is a part of TUServer
 * 
 * TUServer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version
 */

var
	fs = require("fs"),
	http = require("http");

module.exports = {
	//Secondary variables
		server: http.createServer(),
		
		eventHandlers: {},
	
	//Methods
		run: function(host, port) {
			this.server.listen(
				port,
				host,
				
				function() {
					console.log(
						"Server started on " +
						module.exports.server.address().address + ":" +
						module.exports.server.address().port
					);
				}
			);
		},
		stop: function() {
			this.server.close(function() {
				console.log(
					"Server stopped"
				);
			});
		},
		
		requestHandler: function(request, response) {
			response.writeHead(200, {
				"Content-Type": "application/json",
				"Access-Control-Allow-Origin": "*"
			});
			
			if (request.method == "POST") {
				var rawData = "";
				request.on("data", function(data) {
					rawData += data;
				});
				
				request.on("end", function() {
					var data = null;
					try {
						data = JSON.parse(rawData);
					} catch(e) {}
					
					if (data instanceof Object) {
						if (data.event) {
							if (data.event in module.exports.eventHandlers) {
								response.end(JSON.stringify(
									module.exports.eventHandlers[data.event](data)
								));
								
								return;
							}
						}
					}
					
					response.end(JSON.stringify({
						error: "event doesn't exist"
					}));
				});
			} else {
				response.end(JSON.stringify({
					error: "invalid method"
				}));
			}
		},
		
		on: function(event, handler) {
			this.eventHandlers[event] = handler;
		},
		emit: function(event, data) {
			this.eventHandlers[event](data);
		}
};

module.exports.server.on("request", module.exports.requestHandler);
